/* 
 *   Creation Date: <2003/05/25 19:31:42 samuel>
 *   Time-stamp: <2003/05/27 17:43:26 samuel>
 *   
 *	<test.c>
 *	
 *	MPC107 testing
 *   
 *   Copyright (C) 2003 Samuel Rydh (samuel@ibrium.se)
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   version 2
 *   
 */

#include "archinclude.h"

int
alpha( int argc, char **argv ) 
{
	return 0;
}
void
xx_store_pte_lowmem( void )
{
}

void *
vmalloc_mol( int size )
{
	return NULL;
}

void
vfree_mol( void *p )
{
}

void *
kmalloc_mol( int size )
{
	return NULL;
}
void
kfree_mol( void *p )
{
}

void *
kmalloc_cont_mol( int size )
{
	return NULL;
}

void
kfree_cont_mol( void *p )
{
}

ulong
alloc_page_mol( void )
{
	return NULL;
}

void
free_page_mol( ulong p )
{
}

uint
copy_to_user_mol( void *to, const void *from, ulong len )
{
	return 1;
}

uint
copy_from_user_mol( void *to, const void *from, ulong len )
{
	return 1;
}
